/************************************************** -*- tab-width:4 -*- *
* "Epsilon" is a registered trademark licensed to Lugaru Software, Ltd. *
*		"EEL" and "Lugaru" are trademarks of Lugaru Software, Ltd.		*
*																		*
*  Copyright (C) 1996, 2020 Lugaru Software Ltd.  All rights reserved.  *
*																		*
* Limited permission is hereby granted to reproduce and modify this		*
* copyrighted material provided that the resulting code is used only in *
* conjunction with Lugaru products and that this notice is retained in	*
* any such reproduction or modification.								*
************************************************************************/

// Fix bugs in 14.03

#include "eel.h"
#include "grep.h"
#include "dired.h"
#include "c.h"
#include "epsnet.h"

// Is the character just before point in a comment or string? Return
// its code.
int get_prev_char_comment_code(int c_like)
{
	save_var point--;
	return c_like ? in_c_comment(c_look_back) : in_comment();
}

// We're inserting the character in key; maybe break this line first
// (or instead).  Return 1 if we still want to insert the character, or
// 0 to ignore it.

// If c_like, we use in_c_comment() and so forth, even if major_mode
// says it's not exactly C (i.e. Perl mode).  If !c_like, use generic
// code instead.

int do_maybe_break_line(int c_like)
{
	int comm, res = 1, starts_with_comm, starts_in_comm, after_prefix;
	int comm_start = -1, break_block_comment = 0;
	int generic = strcmp(major_mode, "C");
	char fill[FNAMELEN], pat[FNAMELEN], raw_comm[FNAMELEN];
	spot after, next_ends = 0;

	delete_if_highlighted();
	if (c_like && !in_perl_buffer)
		generic = 0;

	// Various reasons we might want not want to fill right now.
	if (give_end_line() - give_begin_line() > FNAMELEN - 10)
		return 1;			// The line's too long for us.
	if (IS_ENTER_KEY(key)
		  && get_prev_char_comment_code(c_like) == IN_OLD_COMMENT
		  && (auto_fill_comment_rules & 1))
		break_block_comment = 1, res = 0;
	else if (current_column() + 1 <= margin_right)
		return 1;		// We're not past the margin.
	comm = get_prev_char_comment_code(c_like);
	if (!(comm & IN_COMMENT))
		return 1;			// We're not in a comment.

	// Collect info on the situation.
	if (key == ' ') {		// Insert <Space> now, then remember not
		insert(key);		// to later, so we can break here.
		res = 0;
	}
	after = alloc_spot(1);	// Remember where we should end up.
	to_begin_line();	// See if a comment's the first thing on this line.
	strcpy(raw_comm, get_comment_start());
	sprintf(pat, "[ \t]*%s", generic ? raw_comm : "/[*/]");
	starts_with_comm = parse_string(1, pat);	// Yes, comment's first on line.
	starts_in_comm = c_like ? in_c_comment(c_look_back) : in_comment();
	// Does this line start with a comment?
	sprintf(pat, "[ \t%s]*", generic ? raw_comm : "*/");
	parse_string(1, pat, fill);	// Repeat this prefix on new lines.
	after_prefix = point;
	if (!starts_in_comm) {		// Find where this line's comment starts.
		to_begin_line();
		sprintf(pat, "<min>.*%s",
				generic ? raw_comm : comm == IN_NEW_COMMENT ? "//" : "/%*");
		point += parse_string(1, pat);
		comm_start = point;
	}
	if (nl_forward()) {		// Remember the end of the next line, unless
		to_end_line();		// there's a comment there.
		if (c_like && !in_c_comment(c_look_back))
			next_ends = alloc_spot(0);
	}

	// Look for a breaking point.
	point = *after;
	if ((c_like && c_auto_fill_mode != 1 || starts_with_comm || starts_in_comm)
		  && (auto_fill_comment_rules & 2) && !break_block_comment
		  && pick_break(margin_right) && point > after_prefix
		  && point > comm_start)
		break_block_comment = 1;

	if (break_block_comment)
		comm_break_line_here(c_like, comm, starts_with_comm, starts_in_comm,
							 comm_start, next_ends, after, fill);

	point = *after;
	free_spot(after);
	if (next_ends)
		free_spot(next_ends);
	return res;
}

do_scp_xfer(struct url_parts *parts, struct scp_job *p, char *template,
			int mode, int use_tmp)
{
	char tmpfile[FNAMELEN], cmd[FNAMELEN], *args[5], *s, cmdname[FNAMELEN];

	if (use_tmp) {
		new_file_write("", translation_type, NULL, FILE_IO_TEMPFILE, mode);
		strcpy(p->file, get_tempfile_name());
		p->flags |= SCP_TMP_FILE;
		delete_file(p->file);
		// Patch issue with ~ character, Swedish language, and inherit.exe.
		while (s = strchr(p->file, '~'))
			*s = 'x';
		// Instead of using FILE_IO_TEMPFILE to get an unused name, for now just pick one and
		// hope it's ok.
		sprintf(p->file + strlen(p->file), "%d", time_ms());
	}
	if (!(p->flags & (SCP_SCP_XFER | SCP_CLIENT_MODE))
		  && !strchr(p->file, '%')) {
		scp_run_helper(parts, p);
		p->flags |= SCP_IS_XFER;
		sprintf(cmd, (p->op == FTP_SEND) ? "put \"%s\" \"%%s\""
										 : "get \"%%s\" \"%s\"", p->file);
		if (is_mswindows)
			for (s = cmd; s = strchr(s, '\\'); )
				*s = '/';	// Windows sftp gets confused by \'s sometimes.
		scp_send_some_command(parts, p, cmd);
		return;
	}
	strcpy(tmpfile, get_tail(p->file, 1));
	args[0] = parts->usr;
	args[1] = parts->host;
	args[2] = parts->fname;
	args[3] = tmpfile;
	ssh_command_variable(cmdname, "sftp", "scp-%s-sftp-command", parts, 1);
	args[4] = cmdname;
	if (expand_string_template(cmd, template, "uhflc", args))
		error("Invalid scp template.");
	scp_start_process(p, cmd);
}

